package com.fjhx.sale.service.after.impl;

import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.customer.service.customer.CustomerService;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.sale.entity.after.dto.AfterSalesDetailSelectDto;
import com.fjhx.sale.entity.after.po.AfterSalesDetail;
import com.fjhx.sale.entity.after.vo.AfterSalesDetailVo;
import com.fjhx.sale.mapper.after.AfterSalesDetailMapper;
import com.fjhx.sale.service.after.AfterSalesDetailService;
import com.fjhx.sale.service.contract.ContractService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * <p>
 * 售后明细 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-20
 */
@Service
public class AfterSalesDetailServiceImpl extends ServiceImpl<AfterSalesDetailMapper, AfterSalesDetail> implements AfterSalesDetailService {

    @Autowired
    private CustomerService customerService;
    @Autowired
    private ContractService contractService;
    @Autowired
    private ProductInfoService productInfoService;

    @Override
    public Page<AfterSalesDetailVo> getPage(AfterSalesDetailSelectDto dto) {
        IWrapper<AfterSalesDetail> wrapper = getWrapper();
        //客户id过滤
        wrapper.eq("as1", AfterSalesDetailVo::getCustomerId, dto.getCustomerId());
        //售后类型过滤
        wrapper.eq("as1", AfterSalesDetailVo::getType, dto.getType());
        //售后状态过滤
        wrapper.eq("as1", AfterSalesDetailVo::getStatus, dto.getStatus());
        //关键字搜索
        if (ObjectUtils.isNotEmpty(dto.getKeyword())) {
            List<Long> pIds = productInfoService.listObject(ProductInfo::getId, q -> q
                    .like(ProductInfo::getCode, dto.getKeyword())
                    .like(ProductInfo::getName, dto.getKeyword())
                    .like(ProductInfo::getSpec, dto.getKeyword())
            );
            pIds.add(null);
            wrapper.in("asd", AfterSalesDetail::getProductId, pIds);
        }
        //根据创建时间倒序
        wrapper.orderByDesc("as1", AfterSalesDetail::getCreateTime);
        Page<AfterSalesDetailVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<AfterSalesDetailVo> records = page.getRecords();
        //赋值客户名称
        customerService.attributeAssign(records, AfterSalesDetailVo::getCustomerId, (item, customer) -> {
            item.setCustomerName(customer.getName());
        });
        //赋值合同编号
        contractService.attributeAssign(records, AfterSalesDetailVo::getContractId, (item, contract) -> {
            item.setContractCode(contract.getCode());
        });
        //赋值产品信息
        productInfoService.attributeAssign(records, AfterSalesDetailVo::getProductId, (item, productInfo) -> {
            item.setProductCode(productInfo.getCode());
            item.setProductName(productInfo.getName());
            item.setProductUnit(productInfo.getSpec());
            item.setProductUnit(productInfo.getUnit());
        });
        return page;
    }

}
